@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -66,6 +66,15 @@ t_audio_io* t_audio_io::open(const t_audio_device& dev, bool playback, bool capt
 
 t_audio_io::~t_audio_io() {}
 
+int t_audio_io::get_sample_rate(void) const {
+	return _sample_rate;
+}
+
+bool t_audio_io::open(const string& device, bool playback, bool capture, bool blocking, int channels, t_audio_sampleformat format, int sample_rate, bool short_latency)
+{
+	_sample_rate = sample_rate;
+}
+
 t_oss_io::t_oss_io() : fd(-1), rec_buffersize(0), play_buffersize(0) {
 }
 
@@ -79,7 +88,11 @@ t_oss_io::~t_oss_io()
 	fd = -1;
 }
 
-bool t_oss_io::open(const string& device, bool playback, bool capture, bool blocking, int channels, t_audio_sampleformat format, int sample_rate, bool short_latency) {
+bool t_oss_io::open(const string& device, bool playback, bool capture, bool blocking, int channels, t_audio_sampleformat format, int sample_rate, bool short_latency) 
+{
+	t_audio_io::open(device, playback, capture, blocking, channels, format, sample_rate,
+		short_latency);
+
 	int mode = 0;
 	int status;
 	
@@ -141,7 +154,25 @@ bool t_oss_io::open(const string& device, bool playback, bool capture, bool bloc
 	// Set fragment size
 	int arg;
 	if (short_latency) {
-		arg = 0x00ff0007; // 255 buffers of 2^7 bytes each
+		switch (sys_config->get_oss_fragment_size()) {
+		case 16:
+			arg = 0x00ff0004; // 255 buffers of 2^4 bytes each
+			break;		
+		case 32:
+			arg = 0x00ff0005; // 255 buffers of 2^5 bytes each
+			break;
+		case 64:
+			arg = 0x00ff0006; // 255 buffers of 2^5 bytes each
+			break;
+		case 128:
+			arg = 0x00ff0007; // 255 buffers of 2^7 bytes each
+			break;
+		case 256:
+			arg = 0x00ff0008; // 255 buffers of 2^8 bytes each
+			break;
+		default:
+			arg = 0x00ff0007; // 255 buffers of 2^7 bytes each
+		}
 	} else {
 		arg = 0x00ff000a; // 255 buffers of 2^10 bytes each
 	}
@@ -164,14 +195,20 @@ bool t_oss_io::open(const string& device, bool playback, bool capture, bool bloc
 		msg += strerror(errno);
 		log_file->write_report(msg, "t_oss_io::open",
 			LOG_NORMAL, LOG_CRITICAL);
-		ui->cb_display_msg("Sound card cannot be set to mono.", MSG_CRITICAL);
+		msg = "Sound card cannot be set to ";
+		msg += int2str(channels);
+		msg += " channels.";
+		ui->cb_display_msg(msg, MSG_CRITICAL);
 		return false;
 	}
-	if (arg != 1) {
-		log_file->write_report("Unable to set mono mode",
-			"t_audio_session::open_dsp_full_duplex",
+	if (arg != channels) {
+		log_file->write_report("Unable to set channels",
+			"t_oss_io::open",
 			LOG_NORMAL, LOG_CRITICAL);
-		ui->cb_display_msg("Sound card cannot be set to mono.", MSG_CRITICAL);
+		string msg = "Sound card cannot be set to ";
+		msg += int2str(channels);
+		msg += " channels.";
+		ui->cb_display_msg(msg, MSG_CRITICAL);
 		return false;
 	}
 	// Sample format
@@ -233,7 +270,7 @@ bool t_oss_io::open(const string& device, bool playback, bool capture, bool bloc
 		log_file->write_report(msg, "t_oss_io::open",
 			LOG_NORMAL, LOG_CRITICAL);
 		msg = "Cannot set sound card sample rate to ";
-		msg += int2str(AUDIO_SAMPLE_RATE);
+		msg += int2str(sample_rate);
 		ui->cb_display_msg(msg, MSG_CRITICAL);
 		return false;
 	}
@@ -261,13 +298,12 @@ void t_oss_io::flush(bool playback_buffer, bool recording_buffer) {
 	for (int i = 0; i < 2; i++) {
 		// i == 0: flush playback buffer, 1: flush recording buffer
 		if (i == 0 && playback_buffer || i == 1 && recording_buffer) {
-			audio_buf_info dsp_info;
 			int skip_bytes = ( (i==0) ? play_buffersize : 
 				rec_buffersize) - get_buffer_space(i == 1);
 			if(skip_bytes <= 0) continue;
-			unsigned char *trash = new unsigned char[dsp_info.bytes];
+			unsigned char *trash = new unsigned char[skip_bytes];
 			MEMMAN_NEW_ARRAY(trash);
-			read(trash, dsp_info.bytes);
+			read(trash, skip_bytes);
 			MEMMAN_DELETE_ARRAY(trash);
 			delete [] trash;
 		}
@@ -312,6 +348,8 @@ t_alsa_io::~t_alsa_io() {
 		log_file->write_endl();
 		log_file->write_footer();
 		
+		// Without the snd_pcm_hw_free, snd_pcm_close sometimes fails.
+		snd_pcm_hw_free(pcm_play_ptr);
 		snd_pcm_close(pcm_play_ptr);
 		pcm_play_ptr = 0;
 	}
@@ -322,13 +360,18 @@ t_alsa_io::~t_alsa_io() {
 		log_file->write_endl();
 		log_file->write_footer();
 		
+		snd_pcm_hw_free(pcm_rec_ptr);
 		snd_pcm_close(pcm_rec_ptr);
 		pcm_rec_ptr = 0;
 	}
 }
 
 
-bool t_alsa_io::open(const string& device, bool playback, bool capture, bool blocking, int channels, t_audio_sampleformat format, int sample_rate, bool short_latency) {
+bool t_alsa_io::open(const string& device, bool playback, bool capture, bool blocking, int channels, t_audio_sampleformat format, int sample_rate, bool short_latency) 
+{
+	t_audio_io::open(device, playback, capture, blocking, channels, format, sample_rate,
+		short_latency);
+		
 	int mode = 0;
 	int status;
 	string msg;
@@ -394,6 +437,9 @@ open_again:
 		log_file->write_endl();
 		log_file->write_footer();
 		
+		// Do not call snd_pcm_hw_free here. There is no hardware to release
+		// yet. On ALSA 1.0.9 it is fine to call snd_pcm_hw_free here. But
+		// ALSA 1.0.6 gives an assert.
 		snd_pcm_close(pcm_ptr);
 		mode &= ~SND_PCM_NONBLOCK;
 		goto open_again;
@@ -475,6 +521,11 @@ open_again:
 		}
 	}
 	
+	// Read back card rate for reporting in the log file.
+	unsigned int card_rate;
+	int card_dir;
+	snd_pcm_hw_params_get_rate(hw_params, &card_rate, &card_dir);
+	
 	if ((err = snd_pcm_hw_params_set_channels (pcm_ptr, hw_params, channels)) < 0) {
 		string s("snd_pcm_hw_params_set_channels(");
 		s += int2str(channels);
@@ -489,7 +540,11 @@ open_again:
 	
 	// Set the size of one period in samples
 	if (short_latency) {
-		buffersize = 128;
+		if (playback) {
+			buffersize = sys_config->get_alsa_play_period_size();
+		} else {
+			buffersize = sys_config->get_alsa_capture_period_size();
+		}
 	} else {
 		buffersize = 1024;
 	}
@@ -499,9 +554,17 @@ open_again:
 		HANDLE_ALSA_ERROR("snd_pcm_hw_params_set_period_size_near");
 	}
 	
-	if(buffersize < 256) {
-		periods *= 2;
+	// The number of periods determines the ALSA application buffer size.
+	// This size must be larger than the jitter buffer.
+	// TODO: use some more sophisticated algorithm here: read back the period
+	//       size and calculate the number of periods needed (only in the
+	//       short latency case)?
+	if (buffersize <= 64) {
+		periods *= 8;
+	} else if (buffersize <= 256) {
+		periods *= 4;
 	}
+	
 	dir = 1;
 	if ((err = snd_pcm_hw_params_set_periods(pcm_ptr, hw_params, periods, dir)) < 0) {
 		if ((err = snd_pcm_hw_params_set_periods_near(pcm_ptr, hw_params, 
@@ -548,6 +611,9 @@ open_again:
 		
 		log_file->write_header("t_alsa_io::open", LOG_NORMAL, LOG_DEBUG);
 		log_file->write_raw("ALSA playback buffer settings.\n");
+		log_file->write_raw("Rate = ");
+		log_file->write_raw(card_rate);
+		log_file->write_raw(" frames/sec\n");
 		log_file->write_raw("Frame size = ");
 		log_file->write_raw(play_framesize);
 		log_file->write_raw(" bytes\n");
@@ -590,6 +656,9 @@ open_again:
 		
 		log_file->write_header("t_alsa_io::open", LOG_NORMAL, LOG_DEBUG);
 		log_file->write_raw("ALSA capture buffer settings.\n");
+		log_file->write_raw("Rate = ");
+		log_file->write_raw(card_rate);
+		log_file->write_raw(" frames/sec\n");
 		log_file->write_raw("Frame size = ");
 		log_file->write_raw(rec_framesize);
 		log_file->write_raw(" bytes\n");
@@ -625,8 +694,23 @@ void t_alsa_io::enable(bool enable_playback, bool enable_recording) {
 }
 
 void t_alsa_io::flush(bool playback_buffer, bool recording_buffer) {
-	if (playback_buffer && pcm_play_ptr) snd_pcm_reset(pcm_play_ptr);
-	if (recording_buffer && pcm_rec_ptr) snd_pcm_reset(pcm_rec_ptr);
+	if (playback_buffer && pcm_play_ptr) {
+		// snd_pcm_reset(pcm_play_ptr);
+		snd_pcm_drop(pcm_play_ptr);
+		snd_pcm_prepare(pcm_play_ptr);
+		snd_pcm_start(pcm_play_ptr);
+	}
+	if (recording_buffer && pcm_rec_ptr) {
+		// For some obscure reason snd_pcm_reset causes the CPU
+		// load to rise to 99.9% when playing and capturing is
+		// done on the same sound card.
+		// Therefor snd_pcm_reset is replaced by functions to
+		// stop the card, drop samples and start again.
+		// snd_pcm_reset(pcm_rec_ptr);
+		snd_pcm_drop(pcm_rec_ptr);
+		snd_pcm_prepare(pcm_rec_ptr);
+		snd_pcm_start(pcm_rec_ptr);
+	}
 }
 
 int t_alsa_io::get_buffer_space(bool is_recording_buffer) {
@@ -705,7 +789,7 @@ int t_alsa_io::read(unsigned char* buf, int len) {
 			snd_pcm_prepare(pcm_rec_ptr);
 			snd_pcm_start(pcm_rec_ptr);
 			continue;
-		} else if (read < 0) {
+		} else if (read <= 0) {
 			msg = "PCM read error: ";
 			msg += snd_strerror(read);
 			log_file->write_report(msg, "t_alsa_io::read", LOG_NORMAL, LOG_DEBUG);
