@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,11 +24,14 @@
 #include <sys/time.h>
 #include "audio_tx.h"
 #include "log.h"
+#include "phone.h"
 #include "userintf.h"
 #include "line.h"
 #include "audits/memman.h"
 
-#define SAMPLE_BUF_SIZE (MAX_PTIME * AUDIO_SAMPLE_RATE/1000 * AUDIO_SAMPLE_SIZE/8)
+extern t_phone *phone;
+
+#define SAMPLE_BUF_SIZE (MAX_PTIME * sc_sample_rate/1000 * AUDIO_SAMPLE_SIZE/8)
 
 // Debug macro to print timestamp
 #define DEBUG_TS(s)	{ gettimeofday(&debug_timer, NULL);\
@@ -38,55 +41,58 @@
 			  cout << " " << (s) << endl;\
 			}
 
-//////////
-// PRIVATE
-//////////
-
-short t_audio_tx::decode(unsigned char sample) {
-	switch(codec) {
-	case CODEC_G711_ALAW:
-		return (short)alaw2linear(sample);
-	case CODEC_G711_ULAW:
-		return (short)ulaw2linear(sample);
-	default:
-		assert(false);
-	}
-}
-
 //////////
 // PUBLIC
 //////////
 
 t_audio_tx::t_audio_tx(t_audio_session *_audio_session,
 		   t_audio_io *_playback_device, t_twinkle_rtp_session *_rtp_session,
-	           t_audio_codec _codec, unsigned short _ptime)
+	           t_audio_codec _codec, 
+	           const map<unsigned short, t_audio_codec> &_payload2codec,
+	           unsigned short _ptime)
 {
 	audio_session = _audio_session;
+	
+	user_config = audio_session->get_line()->get_user();
+	assert(user_config);
+	
 	playback_device = _playback_device;
 	rtp_session = _rtp_session;
 	codec = _codec;
+	sc_sample_rate = audio_sample_rate(_codec);
+	payload2codec = _payload2codec;
 	is_running = false;
 	stop_running = false;
+	
+	// Create audio decoders
+	map_audio_decoder[CODEC_G711_ALAW] = new t_g711a_audio_decoder(_ptime, user_config);
+	MEMMAN_NEW(map_audio_decoder[CODEC_G711_ALAW]);
+	
+	map_audio_decoder[CODEC_G711_ULAW] = new t_g711u_audio_decoder(_ptime, user_config);
+	MEMMAN_NEW(map_audio_decoder[CODEC_G711_ULAW]);
+	
+	map_audio_decoder[CODEC_GSM] = new t_gsm_audio_decoder(user_config);
+	MEMMAN_NEW(map_audio_decoder[CODEC_GSM]);
+	
+#ifdef HAVE_SPEEX
+	map_audio_decoder[CODEC_SPEEX_NB] = new t_speex_audio_decoder(
+			t_speex_audio_decoder::MODE_NB, user_config);
+	MEMMAN_NEW(map_audio_decoder[CODEC_SPEEX_NB]);
 
-	// Determine the expected ptime. The ptime of received packets
-	// might differ.
-	if (codec == CODEC_GSM) {
-		// GSM has fixed ptime
-		ptime = PTIME_GSM;
-	} else if (_ptime == 0) {
-		switch(codec) {
-		case CODEC_G711_ALAW:
-			ptime = PTIME_G711_ALAW;
-			break;
-		case CODEC_G711_ULAW:
-			ptime = PTIME_G711_ULAW;
-			break;
-		default:
-			assert(false);
-		}
-	} else {
-		ptime = _ptime;
-	}
+	map_audio_decoder[CODEC_SPEEX_WB] = new t_speex_audio_decoder(
+			t_speex_audio_decoder::MODE_WB, user_config);
+	MEMMAN_NEW(map_audio_decoder[CODEC_SPEEX_WB]);
+	
+	map_audio_decoder[CODEC_SPEEX_UWB] = new t_speex_audio_decoder(
+			t_speex_audio_decoder::MODE_UWB, user_config);
+	MEMMAN_NEW(map_audio_decoder[CODEC_SPEEX_UWB]);
+#endif
+#ifdef HAVE_ILBC
+	map_audio_decoder[CODEC_ILBC] = new t_ilbc_audio_decoder(_ptime, user_config);
+	MEMMAN_NEW(map_audio_decoder[CODEC_ILBC]);
+#endif
+
+	ptime = map_audio_decoder[codec]->get_default_ptime();
 
 	sample_buf = new unsigned char[SAMPLE_BUF_SIZE];
 	MEMMAN_NEW_ARRAY(sample_buf);
@@ -101,14 +107,11 @@ t_audio_tx::t_audio_tx(t_audio_session *_audio_session,
 	conceal_pos = 0;
 
 	// Initialize jitter buffer
-	jitter_buf = new unsigned char[JITTER_BUF_SIZE];
+	jitter_buf = new unsigned char[JITTER_BUF_SIZE(sc_sample_rate)];
 	MEMMAN_NEW_ARRAY(jitter_buf);
 	jitter_buf_len = 0;
 	load_jitter_buf = true;
 	soundcard_buf_size = playback_device->get_buffer_size(false);
-	
-	// Create GSM decoder
-	gsm_decoder = gsm_create();
 
 	// Initialize 3-way settings
 	is_3way = false;
@@ -117,6 +120,10 @@ t_audio_tx::t_audio_tx(t_audio_session *_audio_session,
 	peer_tx_3way = NULL;
 	peer_rx_3way = NULL;
 	mix_buf_3way = NULL;
+	
+	// Initialize telephone event settings
+	pt_telephone_event = -1;
+	pt_telephone_event_alt = 1;
 }
 
 t_audio_tx::~t_audio_tx() {
@@ -141,9 +148,14 @@ t_audio_tx::~t_audio_tx() {
 		MEMMAN_DELETE_ARRAY(conceal_buf[i]);
 		delete [] conceal_buf[i];
 	}
-
-	// Destroy GSM decoder
-	gsm_destroy(gsm_decoder);
+	
+	// Destroy audio decoders
+	for (map<t_audio_codec, t_audio_decoder *>::iterator i = map_audio_decoder.begin();
+	     i != map_audio_decoder.end(); i++)
+	{
+		MEMMAN_DELETE(i->second);
+		delete i->second;
+	}
 
 	// Cleanup 3-way resources
 	if (media_3way_peer_tx) {
@@ -178,6 +190,25 @@ void t_audio_tx::retain_for_concealment(unsigned char *buf, unsigned short len)
 }
 
 void t_audio_tx::conceal(short num) {
+	// Some codecs have a PLC.
+	// Only use this PLC is the sound card sample rate equals the codec
+	// sample rate. If they differ, then we should resample the codec
+	// samples. As this should be a rare case, we are lazy here. In
+	// this rare case, use Twinkle's low-tech PLC.
+	if (map_audio_decoder[codec]->has_plc() && audio_sample_rate(codec) == sc_sample_rate) {
+		short *sb = (short *)sample_buf;
+		for (int i = 0; i < num; i++) {
+			int nsamples;
+			nsamples = map_audio_decoder[codec]->conceal(sb, SAMPLE_BUF_SIZE);
+			if (nsamples > 0) {
+				play_pcm(sample_buf, nsamples * 2);
+			}
+		}
+		
+		return;
+	}
+
+	// Replay previous packets for other codecs
 	short i = (conceal_pos + (MAX_CONCEALMENT - num)) % MAX_CONCEALMENT;
 
 	if (i >= conceal_pos) {
@@ -221,14 +252,14 @@ void t_audio_tx::play_pcm(unsigned char *buf, unsigned short len, bool only_3rd_
 	// If we are in a 3-way then send the samples to the peer audio
 	// receiver for mixing
 	if (!only_3rd_party && is_3way && peer_rx_3way) {
-		peer_rx_3way->post_media_peer_tx_3way(buf, len);
+		peer_rx_3way->post_media_peer_tx_3way(buf, len, sc_sample_rate);
 	}
 
 	// If we are in a 3-way conference and we are not the mixer then
 	// send the sound samples to the mixer
 	if (is_3way && !is_3way_mixer) {
 		if (peer_tx_3way) {
-			peer_tx_3way->post_media_peer_tx_3way(buf, len);
+			peer_tx_3way->post_media_peer_tx_3way(buf, len, sc_sample_rate);
 			return;
 		} else {
 			// There is no peer.
@@ -251,7 +282,7 @@ void t_audio_tx::play_pcm(unsigned char *buf, unsigned short len, bool only_3rd_
 
 	// Fill jitter buffer before playing
 	if (load_jitter_buf) {
-		if (jitter_buf_len + len < JITTER_BUF_SIZE) {
+		if (jitter_buf_len + len < JITTER_BUF_SIZE(sc_sample_rate)) {
 			memcpy(jitter_buf + jitter_buf_len, playbuf, len);
 			jitter_buf_len += len;
 		} else {
@@ -285,14 +316,14 @@ void t_audio_tx::play_pcm(unsigned char *buf, unsigned short len, bool only_3rd_
 	// to be refilled. This should only occur when no RTP packets
 	// have been received for a while (silence suppression or packet loss)
 	int bufferspace = playback_device->get_buffer_space(false);
-	if (bufferspace == soundcard_buf_size && len <= JITTER_BUF_SIZE) {
+	if (bufferspace == soundcard_buf_size && len <= JITTER_BUF_SIZE(sc_sample_rate)) {
 		memcpy(jitter_buf, playbuf, len);
 		jitter_buf_len = len;
 		load_jitter_buf = true;
 		log_file->write_header("t_audio_tx::play_pcm", LOG_NORMAL, LOG_DEBUG);
 		log_file->write_raw("Audio tx line ");
 		log_file->write_raw(get_line()->get_line_number()+1);
-		log_file->write_raw(": jitter buffer empy.\n");
+		log_file->write_raw(": jitter buffer empty.\n");
 		log_file->write_footer();
 		return;
 	}
@@ -303,6 +334,12 @@ void t_audio_tx::play_pcm(unsigned char *buf, unsigned short len, bool only_3rd_
 	// This can only happen if the thread did not get
 	// processing time for a while and RTP packets start to
 	// pile up.
+	// Or if a soundcard plays out the samples at just less then
+	// the requested sample rate.
+	/* Not needed anymore, the ::run loop already discards incoming RTP packets
+	   with a late timestamp. This seems to solve the slow soundcard problem
+	   better. The solution below caused annoying ticks in the playout.
+	   
 	if (soundcard_buf_size - bufferspace > JITTER_BUF_SIZE + len) {
 		log_file->write_header("t_audio_tx::play_pcm", LOG_NORMAL, LOG_DEBUG);
 		log_file->write_raw("Audio tx line ");
@@ -313,6 +350,7 @@ void t_audio_tx::play_pcm(unsigned char *buf, unsigned short len, bool only_3rd_
 		log_file->write_footer();
 		return;
 	}
+	*/
 
 	// Write passed sound samples to DSP.
 	status = playback_device->write(playbuf, len);
@@ -339,7 +377,12 @@ void t_audio_tx::run(void) {
 	// a crash when the thread gets destroyed before it starts running.
 	// is_running = true;
 
-	unsigned long rtp_timestamp;
+	uint32 rtp_timestamp;
+	
+	// This thread may not take the lock on the transaction layer to
+	// prevent dead locks
+	phone->add_prohibited_thread();
+	ui->add_prohibited_thread();
 	
 	while (true) {
 		do {
@@ -367,11 +410,11 @@ void t_audio_tx::run(void) {
 				// this must be sent to the dsp.
 				if (is_3way && is_3way_mixer &&
 				    media_3way_peer_tx->size_content() >=
-				    	ptime * (AUDIO_SAMPLE_RATE / 1000) * 2)
+				    	ptime * (audio_sample_rate(codec) / 1000) * 2)
 				{
 					// Fill the sample buffer with silence
-					int len = ptime * (AUDIO_SAMPLE_RATE / 1000);
-					bzero(sample_buf, len);
+					int len = ptime * (audio_sample_rate(codec) / 1000);
+					memset(sample_buf, 0, len);
 					play_pcm(sample_buf, len, true);
 				}
 
@@ -403,76 +446,58 @@ void t_audio_tx::run(void) {
 		}
 
 		// Check for a codec change
-		switch (adu->getType()) {
-		case sptPCMU:
-			if (codec != CODEC_G711_ULAW) {
-				codec = CODEC_G711_ULAW;
+		map<unsigned short, t_audio_codec>::const_iterator it_codec;
+		it_codec = payload2codec.find(adu->getType());
+		t_audio_codec recvd_codec = CODEC_NULL;
+		if (it_codec != payload2codec.end()) {
+			recvd_codec = it_codec->second;
+		}
+		
+		map<t_audio_codec, t_audio_decoder *>::const_iterator it_decoder;
+		it_decoder = map_audio_decoder.find(recvd_codec);
+		if (it_decoder != map_audio_decoder.end()) {
+			if (codec != recvd_codec) {
+				codec = recvd_codec;
 				get_line()->ci_set_recv_codec(codec);
-				ui->cb_recv_codec_changed(get_line()->get_line_number(),
+				ui->cb_async_recv_codec_changed(get_line()->get_line_number(),
 					codec);
 
-				log_file->write_header("t_audio_tx::run", LOG_NORMAL, LOG_DEBUG);
+				log_file->write_header("t_audio_tx::run", 
+					LOG_NORMAL, LOG_DEBUG);
 				log_file->write_raw("Audio tx line ");
 				log_file->write_raw(get_line()->get_line_number()+1);
-				log_file->write_raw(": codec change to g711u.\n");
+				log_file->write_raw(": codec change to ");
+				log_file->write_raw(ui->format_codec(codec));
+				log_file->write_endl();
 				log_file->write_footer();
 			}
-			break;
-		case sptPCMA:
-			if (codec != CODEC_G711_ALAW) {
-				codec = CODEC_G711_ALAW;
-				get_line()->ci_set_recv_codec(codec);
-				ui->cb_recv_codec_changed(get_line()->get_line_number(),
-					codec);
-
-				log_file->write_header("t_audio_tx::run", LOG_NORMAL, LOG_DEBUG);
-				log_file->write_raw("Audio tx line ");
-				log_file->write_raw(get_line()->get_line_number()+1);
-				log_file->write_raw(": codec change to g711a.\n");
-				log_file->write_footer();
-			}
-			break;
-		case sptGSM:
-			if (codec != CODEC_GSM) {
-				codec = CODEC_GSM;
-				get_line()->ci_set_recv_codec(codec);
-				ui->cb_recv_codec_changed(get_line()->get_line_number(),
-					codec);
-
-				log_file->write_header("t_audio_tx::run", LOG_NORMAL, LOG_DEBUG);
-				log_file->write_raw("Audio tx line ");
-				log_file->write_raw(get_line()->get_line_number()+1);
-				log_file->write_raw(": codec change to gsm.\n");
-				log_file->write_footer();
-			}
-			break;
-		default:
+		} else {
 			if (adu->getType() == pt_telephone_event ||
-			    adu->getType() == pt_telephone_event_alt) {
+			    adu->getType() == pt_telephone_event_alt) 
+			{
 				recvd_dtmf = true;
-				break;
+			} else {
+				if (codec != CODEC_UNSUPPORTED) {
+					codec = CODEC_UNSUPPORTED;
+					get_line()->ci_set_recv_codec(codec);
+					ui->cb_async_recv_codec_changed(
+						get_line()->get_line_number(), codec);
+	
+					log_file->write_header("t_audio_tx::run", 
+						LOG_NORMAL, LOG_DEBUG);
+					log_file->write_raw("Audio tx line ");
+					log_file->write_raw(get_line()->get_line_number()+1);
+					log_file->write_raw(": payload type ");
+					log_file->write_raw(adu->getType());
+					log_file->write_raw(" not supported\n");
+					log_file->write_footer();
+				}
+	
+				last_seqnum = adu->getSeqNum();
+				MEMMAN_DELETE(const_cast<ost::AppDataUnit*>(adu));
+				delete adu;
+				continue;
 			}
-
-			if (codec != CODEC_UNSUPPORTED) {
-				codec = CODEC_UNSUPPORTED;
-				get_line()->ci_set_recv_codec(codec);
-				ui->cb_recv_codec_changed(get_line()->get_line_number(),
-					codec);
-
-				log_file->write_header("t_audio_tx::run", LOG_NORMAL, LOG_DEBUG);
-				log_file->write_raw("Audio tx line ");
-				log_file->write_raw(get_line()->get_line_number()+1);
-				log_file->write_raw(": payload type ");
-				log_file->write_raw(adu->getType());
-				log_file->write_raw(" not supported\n");
-				log_file->write_footer();
-			}
-
-			last_seqnum = adu->getSeqNum();
-			MEMMAN_DELETE(const_cast<ost::AppDataUnit*>(adu));
-			delete adu;
-			continue;
-			break;
 		}
 
 		// DTMF event
@@ -488,7 +513,7 @@ void t_audio_tx::run(void) {
 				dtmf_previous_timestamp = rtp_timestamp;
 				t_rtp_telephone_event *e =
 					(t_rtp_telephone_event *)adu->getData();
-				ui->cb_dtmf_detected(get_line()->get_line_number(),
+				ui->cb_async_dtmf_detected(get_line()->get_line_number(),
 					e->get_event());
 
 				// Log DTMF event
@@ -508,29 +533,26 @@ void t_audio_tx::run(void) {
 			continue;
 		}
 
-		// Skip packet if the payload size is too big
-		if (adu->getSize() > SAMPLE_BUF_SIZE / 2) {
+		// Discard invalide payload sizes
+		if (!map_audio_decoder[codec]->valid_payload_size(
+				adu->getSize(), SAMPLE_BUF_SIZE / 2))
+		{
 			log_file->write_header("t_audio_tx::run", LOG_NORMAL, LOG_DEBUG);
 			log_file->write_raw("Audio tx line ");
 			log_file->write_raw(get_line()->get_line_number()+1);
 			log_file->write_raw(": RTP payload size (");
 			log_file->write_raw((unsigned long)(adu->getSize()));
-			log_file->write_raw(" bytes) exceeds maximum\n");
+			log_file->write_raw(" bytes) invalid for \n");
+			log_file->write_raw(ui->format_codec(codec));
 			log_file->write_footer();
 			last_seqnum = adu->getSeqNum();
 			MEMMAN_DELETE(const_cast<ost::AppDataUnit*>(adu));
 			delete adu;
 			continue;
 		}
-
-		// Determine received ptime
+		
 		unsigned short recvd_ptime;
-		if (codec == CODEC_GSM) {
-			recvd_ptime = PTIME_GSM;
-		} else {
-			recvd_ptime = adu->getSize() /
-					(AUDIO_SAMPLE_RATE / 1000);
-		}
+		recvd_ptime = map_audio_decoder[codec]->get_ptime(adu->getSize());
 
 		// Log a change of ptime
 		if (ptime != recvd_ptime) {
@@ -545,39 +567,11 @@ void t_audio_tx::run(void) {
 			log_file->write_footer();
 			ptime = recvd_ptime;
 		}
-
-		// Discard invalid GSM payload sizes
-		if (codec == CODEC_GSM && adu->getSize() != 33) {
-			log_file->write_header("t_audio_tx::run", LOG_NORMAL, LOG_DEBUG);
-			log_file->write_raw("Audio tx line ");
-			log_file->write_raw(get_line()->get_line_number()+1);
-			log_file->write_raw(": invalid GSM payload size: ");
-			log_file->write_raw((unsigned long)(adu->getSize()));
-			log_file->write_endl();
-			log_file->write_footer();
-			last_seqnum = adu->getSeqNum();
-			MEMMAN_DELETE(const_cast<ost::AppDataUnit*>(adu));
-			delete adu;
-			continue;
-		}
-
-		short *sb = (short *)sample_buf;
-		unsigned char *payload = const_cast<uint8 *>(adu->getData());
-
-		switch (codec) {
-		case CODEC_G711_ALAW:
-		case CODEC_G711_ULAW:
-			for (int i = 0; i < adu->getSize(); i++) {
-				sb[i] = decode(payload[i]);
-			}
-			break;
-		case CODEC_GSM:
-			gsm_decode(gsm_decoder, payload, sb);
-			break;
-		default:
-			assert(false);
-		}
-
+		
+		// Check for lost packets
+		// This must be done before decoding the received samples as the
+		// speex decoder has its own PLC algorithm for which it needs the decoding
+		// state before decoding the new samples.
 		if (adu->getSeqNum() != (last_seqnum + 1) % 65536 && last_seqnum != -1) {
 			// Packets have been lost
 			int num_lost = adu->getSeqNum() - last_seqnum - 1;
@@ -597,17 +591,71 @@ void t_audio_tx::run(void) {
 			}
 			clear_conceal_buf();
 		}
-
-		// Determine size of sample to play
-		short sample_size;
-		switch (codec) {
-		case CODEC_G711_ALAW:
-		case CODEC_G711_ULAW:
-			sample_size = adu->getSize() * 2;
-			break;
-		case CODEC_GSM:
-			sample_size = 320; // 160 2-byte samples
-			break;
+		
+		// Determine if resampling is needed due to dynamic change to
+		// codec with other sample rate.
+		short downsample_factor = 1;
+		short upsample_factor = 1;
+		if (audio_sample_rate(codec) > sc_sample_rate) {
+			downsample_factor = audio_sample_rate(codec) / sc_sample_rate;
+		} else if (audio_sample_rate(codec) < sc_sample_rate) {
+			upsample_factor = sc_sample_rate / audio_sample_rate(codec);
+		}
+		
+		// Create sample buffer. If no resampling is needed, the sample
+		// buffer from the audio_tx object can be used directly.
+		// Otherwise a temporary sample buffers is created that will
+		// be resampled to the object's sample buffer later.
+		short *sb;
+		int sb_size;
+		if (downsample_factor > 1) {
+			sb_size = SAMPLE_BUF_SIZE / 2 * downsample_factor;
+			sb = new short[sb_size];
+			MEMMAN_NEW_ARRAY(sb);
+		} else if (upsample_factor > 1) {
+			sb_size = SAMPLE_BUF_SIZE / 2;
+			sb = new short[SAMPLE_BUF_SIZE / 2];
+			MEMMAN_NEW_ARRAY(sb);
+		} else {
+			sb_size = SAMPLE_BUF_SIZE / 2;
+			sb = (short *)sample_buf;
+		}
+				
+		// Decode the audio
+		unsigned char *payload = const_cast<uint8 *>(adu->getData());
+		short sample_size; // size in bytes
+		sample_size = 2 * map_audio_decoder[codec]->decode(payload, adu->getSize(),
+					sb, sb_size);
+				
+		// Resample if needed
+		if (downsample_factor > 1) {
+			short *p = sb;
+			sb = (short *)sample_buf;
+			for (int i = 0; i < sample_size / 2; i += downsample_factor) {
+				sb[i / downsample_factor] = p[i];
+			}
+			MEMMAN_DELETE_ARRAY(p);
+			delete [] p;
+			sample_size /= downsample_factor;
+		} else if (upsample_factor > 1) {
+			short *p = sb;
+			sb = (short *)sample_buf;
+			for (int i = 0; i < sample_size / 2; i++) {
+				for (int j = 0; j < upsample_factor; j++) {
+					sb[i * upsample_factor + j] = p[i];
+				}
+			}
+			MEMMAN_DELETE_ARRAY(p);
+			delete [] p;
+			sample_size *= upsample_factor;
+		}
+		
+		// If the decoder deliverd 0 bytes, then it failed
+		if (sample_size == 0) {
+			last_seqnum = adu->getSeqNum();
+			MEMMAN_DELETE(const_cast<ost::AppDataUnit*>(adu));
+			delete adu;	
+			continue;
 		}
 		
 		// Discard packet if we are lacking behind. This happens if the
@@ -616,14 +664,9 @@ void t_audio_tx::run(void) {
 			uint32 last_ts = rtp_session->getLastTimestamp();
 			uint32 diff;
 			
-			if (last_ts >= rtp_timestamp) {
-				diff = last_ts - rtp_timestamp;
-			} else {
-				// Timestamp wrapped around
-				diff = last_ts - rtp_timestamp + 2^64;
-			}
+			diff = last_ts - rtp_timestamp;
 			
-			if (diff > (JITTER_BUF_SIZE / AUDIO_SAMPLE_SIZE) * 8)
+			if (diff > (JITTER_BUF_SIZE(sc_sample_rate) / AUDIO_SAMPLE_SIZE) * 8)
 			{
 				log_file->write_header("t_audio_tx::run", LOG_NORMAL, LOG_DEBUG);
 				log_file->write_raw("Audio tx line ");
@@ -657,6 +700,8 @@ void t_audio_tx::run(void) {
 		// crack in the sound.
 	}
 
+	phone->remove_prohibited_thread();
+	ui->remove_prohibited_thread();
 	is_running = false;
 }
 
@@ -714,7 +759,7 @@ void t_audio_tx::join_3way(bool mixer, t_audio_tx *peer_tx, t_audio_rx *peer_rx)
 	MEMMAN_NEW_ARRAY(mix_buf_3way);
 
 	// See comments in audio_rx.cpp for the size of this buffer.
-	media_3way_peer_tx = new t_media_buffer(JITTER_BUF_SIZE);
+	media_3way_peer_tx = new t_media_buffer(JITTER_BUF_SIZE(sc_sample_rate));
 	MEMMAN_NEW(media_3way_peer_tx);
 
 	mtx_3way.unlock();
@@ -820,7 +865,7 @@ void t_audio_tx::stop_3way(void) {
 	log_file->write_header("t_audio_tx::stop_3way");
 	log_file->write_raw("Audio tx line ");
 	log_file->write_raw(get_line()->get_line_number()+1);
-	log_file->write_raw(": stop 3-way.");
+	log_file->write_raw(": stop 3-way.\n");
 	log_file->write_footer();
 
 	is_3way = false;
@@ -841,15 +886,29 @@ void t_audio_tx::stop_3way(void) {
 	mtx_3way.unlock();
 }
 
-void t_audio_tx::post_media_peer_tx_3way(unsigned char *media, int len) {
+void t_audio_tx::post_media_peer_tx_3way(unsigned char *media, int len,
+		unsigned short peer_sample_rate) 
+{
 	mtx_3way.lock();
 
 	if (!is_3way || !is_3way_mixer) {
 		mtx_3way.unlock();
 		return;
 	}
-
-	media_3way_peer_tx->add(media, len);
+	
+	if (peer_sample_rate != sc_sample_rate) {
+		// Resample media from peer to sample rate of this transmitter
+		int output_len = (len / 2) * sc_sample_rate / peer_sample_rate;
+		short *output_buf = new short[output_len];
+		MEMMAN_NEW_ARRAY(output_buf);
+		int resample_len = resample((short *)media, len / 2, peer_sample_rate,
+					output_buf, output_len, sc_sample_rate);
+		media_3way_peer_tx->add((unsigned char *)output_buf, resample_len * 2);
+		MEMMAN_DELETE_ARRAY(output_buf);
+		delete [] output_buf;
+	} else {
+		media_3way_peer_tx->add(media, len);
+	}
 
 	mtx_3way.unlock();
 }
