@@ -7,7 +7,7 @@
 ** place of a destructor.
 *****************************************************************************/
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -61,7 +61,7 @@ void SelectNicForm::setAsDefault()
 	
 	// Write default to system settings
 	QStringList l = QStringList::split(':', nicListBox->currentText());
-	sys_config->start_user_host = l[1].ascii();
+	sys_config->set_start_user_host(l[1].ascii());
 	string error_msg;
 	if (!sys_config->write_config(error_msg)) {
 		// Failed to write config file
