@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <signal.h>
 #include "exceptions.h"
 #include "line.h"
+#include "log.h"
 #include "sdp/sdp.h"
 #include "util.h"
 #include "user.h"
@@ -40,6 +41,7 @@ t_call_info::t_call_info() {
 void t_call_info::clear(void) {
 	from_uri.set_url("");
 	from_display.clear();
+	from_display_override.clear();
 	from_organization.clear();
 	to_uri.set_url("");
 	to_display.clear();
@@ -53,6 +55,14 @@ void t_call_info::clear(void) {
 	refer_supported = false;
 }
 
+string t_call_info::get_from_display_presentation(void) const {
+	if (from_display_override.empty()) {
+		return from_display;
+	} else {
+		return from_display_override;
+	}
+}
+
 
 ///////////
 // t_line
@@ -188,10 +198,13 @@ void t_line::cleanup(void) {
 		substate = LSSUB_IDLE;
 		is_on_hold = false;
 		is_muted = false;
+		auto_answer = false;
 		call_info.clear();
 		call_history->add_call_record(call_hist_record);
 		call_hist_record.renew();
 		phone->line_cleared(line_number);
+		user_config = NULL;
+		user_defined_ringtone.clear();
 		ui->cb_line_state_changed();
 	}
 }
@@ -213,12 +226,15 @@ void t_line::cleanup_open_pending(void) {
 	if (!active_dialog) {
 		is_on_hold = false;
 		is_muted = false;
+		auto_answer = false;
 		state = LS_IDLE;
 		substate = LSSUB_IDLE;
 		call_info.clear();
 		call_history->add_call_record(call_hist_record);
 		call_hist_record.renew();
 		phone->line_cleared(line_number);
+		user_config = NULL;
+		user_defined_ringtone.clear();
 		ui->cb_line_state_changed();
 	}
 }
@@ -240,9 +256,12 @@ t_line::t_line(t_phone *_phone, unsigned short _line_number) {
 	active_dialog = NULL;
 	is_on_hold = false;
 	is_muted = false;
+	auto_answer = false;
 	line_number = _line_number;
 	id_invite_comp = 0;
 	id_no_answer = 0;
+	user_config = NULL;
+	user_defined_ringtone.clear();
 }
 
 t_line::~t_line() {
@@ -323,7 +342,7 @@ void t_line::start_timer(t_line_timer timer, t_dialog_id did) {
 		id_invite_comp = t->get_id();
 		break;
 	case LTMR_NO_ANSWER:
-		t = new t_tmr_line(DUR_NO_ANSWER, timer, this, did);
+		t = new t_tmr_line(DUR_NO_ANSWER(user_config), timer, this, did);
 		MEMMAN_NEW(t);
 		id_no_answer = t->get_id();
 		break;
@@ -364,6 +383,12 @@ void t_line::start_timer(t_line_timer timer, t_dialog_id did) {
 		MEMMAN_NEW(t);
 		dialog->id_100rel_guard = t->get_id();
 		break;
+	case LTMR_CANCEL_GUARD:
+		assert(dialog);
+		t = new t_tmr_line(DUR_CANCEL_GUARD, timer, this, did);
+		MEMMAN_NEW(t);
+		dialog->id_cancel_guard = t->get_id();
+		break;		
 	default:
 		assert(false);
 	}
@@ -410,6 +435,19 @@ void t_line::stop_timer(t_line_timer timer, t_dialog_id did) {
 		assert(dialog);
 		id = &dialog->id_100rel_guard;
 		break;
+	case LTMR_CANCEL_GUARD:
+		assert(dialog);
+		id = &dialog->id_cancel_guard;
+		
+		// KLUDGE
+		if (*id == 0) {
+			// Cancel is always sent on the open dialog.
+			// The timer is probably stopped from a pending dialog,
+			// therefore the timer is stopped on the wrong dialog.
+			// Check if the open dialog has a CANCEL guard timer.
+			if (open_dialog) id = &open_dialog->id_cancel_guard;
+		}
+		break;
 	default:
 		assert(false);
 	}
@@ -418,25 +456,29 @@ void t_line::stop_timer(t_line_timer timer, t_dialog_id did) {
 	*id = 0;
 }
 
-void t_line::invite(const t_url &to_uri, const string &to_display,
+void t_line::invite(t_user *user, const t_url &to_uri, const string &to_display,
 		const string &subject)
 {
-	invite(to_uri, to_display, subject, t_hdr_referred_by());
+	invite(user, to_uri, to_display, subject, t_hdr_referred_by());
 }
 
-void t_line::invite(const t_url &to_uri, const string &to_display,
+void t_line::invite(t_user *user, const t_url &to_uri, const string &to_display,
 		const string &subject, const t_hdr_referred_by &hdr_referred_by)
 {
+	assert(user);
+	
 	// Ignore if line is not idle
 	if (state != LS_IDLE) {
 		return;
 	}
 
 	assert(!open_dialog);
+	
+	user_config = user;
 
 	call_info.from_uri = create_user_uri();
-	call_info.from_display = user_config->display;
-	call_info.from_organization = user_config->organization;
+	call_info.from_display = user_config->get_display();
+	call_info.from_organization = user_config->get_organization();
 	call_info.to_uri = to_uri;
 	call_info.to_display = to_display;
 	call_info.to_organization.clear();
@@ -515,17 +557,21 @@ void t_line::end_call(void) {
 	if (active_dialog) {
 		substate = LSSUB_RELEASING;
 		ui->cb_line_state_changed();
-		ui->cb_stop_tone(line_number);
+		ui->cb_stop_call_notification(line_number);
 		active_dialog->send_bye();
 		cleanup();
 		return;
 	}
 
 	// Always send the CANCEL on the open dialog.
+	// The pending dialogs will be cleared when the INVITE gets
+	// terminated.
+	// CANCEL is send on the open dialog as the CANCEL must have
+	// the same tags as the INVITE.
 	if (open_dialog) {
 		substate = LSSUB_RELEASING;
 		ui->cb_line_state_changed();
-		ui->cb_stop_tone(line_number);
+		ui->cb_stop_call_notification(line_number);
 		open_dialog->send_cancel(!pending_dialogs.empty());
 		cleanup();
 		return;
@@ -542,9 +588,9 @@ void t_line::end_call(void) {
 	// proper way. Maybe add to dying_dialogs.
 }
 
-void t_line::send_dtmf(char digit) {
+void t_line::send_dtmf(char digit, bool inband) {
 	if (active_dialog && active_dialog->get_state() == DS_CONFIRMED) {
-		active_dialog->send_dtmf(digit);
+		active_dialog->send_dtmf(digit, inband);
 		cleanup();
 		return;
 	}
@@ -717,13 +763,15 @@ void t_line::recvd_success(t_response *r, t_tuid tuid, t_tid tid) {
 
 void t_line::recvd_redirect(t_response *r, t_tuid tuid, t_tid tid) {
 	t_dialog *d;
+	
+	assert(user_config);
 
 	if (active_dialog) {
 		// If an active dialog exists then non-2XX should
 		// only be for this dialog.
 		if (active_dialog->match_response(r, 0)) {
 			// Redirection of mid-dialog request
-			if (!user_config->allow_redirection ||
+			if (!user_config->get_allow_redirection() ||
 			    !active_dialog->redirect_request(r))
 			{
 				// Redirection not allowed/failed
@@ -756,7 +804,7 @@ void t_line::recvd_redirect(t_response *r, t_tuid tuid, t_tid tid) {
 			pending_dialogs.clear();
 
 			if (open_dialog) {
-				if (!user_config->allow_redirection ||
+				if (!user_config->get_allow_redirection() ||
 				    !open_dialog->redirect_invite(r))
 				{
 					MEMMAN_DELETE(open_dialog);
@@ -785,7 +833,7 @@ void t_line::recvd_redirect(t_response *r, t_tuid tuid, t_tid tid) {
 		}
 
 		if (r->hdr_cseq.method == INVITE) {
-			if (!user_config->allow_redirection ||
+			if (!user_config->get_allow_redirection() ||
 			    !open_dialog->redirect_invite(r))
 			{
 				// Redirection failed/not allowed
@@ -817,6 +865,8 @@ void t_line::recvd_redirect(t_response *r, t_tuid tuid, t_tid tid) {
 
 void t_line::recvd_client_error(t_response *r, t_tuid tuid, t_tid tid) {
 	t_dialog *d;
+	
+	assert(user_config);
 
 	if (active_dialog) {
 		// If an active dialog exists then non-2XX should
@@ -838,7 +888,7 @@ void t_line::recvd_client_error(t_response *r, t_tuid tuid, t_tid tid) {
 			if (!response_processed) {
 				// The request failed, redirect it if there
 				// are other destinations available.
-				if (!user_config->allow_redirection ||
+				if (!user_config->get_allow_redirection() ||
 				    !active_dialog->redirect_request(r))
 				{
 					// Request failed
@@ -897,7 +947,7 @@ void t_line::recvd_client_error(t_response *r, t_tuid tuid, t_tid tid) {
 				if (!response_processed) {
 					// The request failed, redirect it if there
 					// are other destinations available.
-					if (!user_config->allow_redirection ||
+					if (!user_config->get_allow_redirection() ||
 					    !open_dialog->redirect_invite(r))
 					{
 						// Request failed
@@ -954,7 +1004,7 @@ void t_line::recvd_client_error(t_response *r, t_tuid tuid, t_tid tid) {
 			if (!response_processed) {
 				// The request failed, redirect it if there
 				// are other destinations available.
-				if (!user_config->allow_redirection ||
+				if (!user_config->get_allow_redirection() ||
 				    !open_dialog->redirect_invite(r))
 				{
 					// Request failed
@@ -986,16 +1036,169 @@ void t_line::recvd_client_error(t_response *r, t_tuid tuid, t_tid tid) {
 }
 
 void t_line::recvd_server_error(t_response *r, t_tuid tuid, t_tid tid) {
-	recvd_redirect(r, tuid, tid);
+	t_dialog *d;
+
+	assert(user_config);
+	
+	if (active_dialog) {
+		// If an active dialog exists then non-2XX should
+		// only be for this dialog.
+		if (active_dialog->match_response(r, 0)) {
+			bool response_processed = false;
+
+			if (r->code == R_503_SERVICE_UNAVAILABLE) {
+				// RFC 3263 4.3
+				// Failover to next destination
+				if (active_dialog->failover_request(r))
+				{
+					// Failover successul.
+					// The response does not need to be
+					// processed any further
+					response_processed = true;
+				}
+			}
+
+			if (!response_processed) {
+				// The request failed, redirect it if there
+				// are other destinations available.
+				if (!user_config->get_allow_redirection() ||
+				    !active_dialog->redirect_request(r))
+				{
+					// Request failed
+					active_dialog->
+						recvd_response(r, tuid, tid);
+				}
+			}
+		}
+
+		cleanup();
+		return;
+	}
+
+	d = match_response(r, pending_dialogs);
+	if (d) {
+		d->recvd_response(r, tuid, tid);
+		if (r->hdr_cseq.method == INVITE) {
+			pending_dialogs.remove(d);
+			MEMMAN_DELETE(d);
+			delete d;
+
+			// RFC 3261 13.2.2.3
+			// All early dialogs are considered terminated
+			// upon reception of the non-2xx final response.
+			list<t_dialog *>::iterator i;
+			for (i = pending_dialogs.begin();
+			     i != pending_dialogs.end(); i++)
+			{
+				MEMMAN_DELETE(*i);
+				delete *i;
+			}
+			pending_dialogs.clear();
+
+			if (open_dialog) {
+				bool response_processed = false;
+
+				if (r->code == R_503_SERVICE_UNAVAILABLE) {
+					// INVITE failover
+					if (open_dialog->failover_invite())
+					{
+						// Failover successul.
+						// The response does not need to
+						// be processed any further
+						response_processed = true;
+					}
+				}
+
+				if (!response_processed) {
+					// The request failed, redirect it if there
+					// are other destinations available.
+					if (!user_config->get_allow_redirection() ||
+					    !open_dialog->redirect_invite(r))
+					{
+						// Request failed
+						MEMMAN_DELETE(open_dialog);
+						delete open_dialog;
+						open_dialog = NULL;
+					}
+				}
+			}
+		}
+
+		cleanup();
+		return;
+	}
+
+	d = match_response(r, dying_dialogs);
+	if (d) {
+		d->recvd_response(r, tuid, tid);
+		cleanup();
+		return;
+	}
+
+	if (open_dialog && open_dialog->match_response(r, tuid)) {
+		// If the response is a 503 then do not send the
+		// response to the dialog as the request must be resent.
+		// For an INVITE request, the transaction layer has already
+		// sent ACK for a failure response.
+		if (r->code != R_503_SERVICE_UNAVAILABLE && r->hdr_cseq.method != INVITE) {
+			open_dialog->recvd_response(r, tuid, tid);
+		}
+
+		if (r->hdr_cseq.method == INVITE) {
+			bool response_processed = false;
+
+			if (r->code == R_503_SERVICE_UNAVAILABLE) {
+				// INVITE failover
+				if (open_dialog->failover_invite())
+				{
+					// Failover successul.
+					// The response does not need to
+					// be processed any further
+					response_processed = true;
+				}
+			}
+
+			if (!response_processed) {
+				// The request failed, redirect it if there
+				// are other destinations available.
+				if (!user_config->get_allow_redirection() ||
+				    !open_dialog->redirect_invite(r))
+				{
+					// Request failed
+					open_dialog->recvd_response(r, tuid, tid);
+					MEMMAN_DELETE(open_dialog);
+					delete open_dialog;
+					open_dialog = NULL;
+				}
+			}
+
+			// RFC 3261 13.2.2.3
+			// All early dialogs are considered terminated
+			// upon reception of the non-2xx final response.
+			list<t_dialog *>::iterator i;
+			for (i = pending_dialogs.begin();
+			     i != pending_dialogs.end(); i++)
+			{
+				MEMMAN_DELETE(*i);
+				delete *i;
+			}
+			pending_dialogs.clear();
+		}
+
+		cleanup();
+		return;
+	}
+
+	// out-of-dialog responses should be handled by the phone
 }
 
 void t_line::recvd_global_error(t_response *r, t_tuid tuid, t_tid tid) {
 	recvd_redirect(r, tuid, tid);
 }
 
-void t_line::recvd_invite(t_request *r, t_tid tid) {
+void t_line::recvd_invite(t_user *user, t_request *r, t_tid tid, const string &ringtone) {
 	t_response *resp;
-
+	
 	switch (state) {
 	case LS_IDLE:
 		assert(!active_dialog);
@@ -1019,18 +1222,14 @@ void t_line::recvd_invite(t_request *r, t_tid tid) {
 			return;
 		}
 		*/
-
-		// Check user in the request-URI
-		if (r->uri.get_user() != user_config->name) {
-			resp = r->create_response(R_404_NOT_FOUND);
-			send_response(resp, 0, tid);
-			MEMMAN_DELETE(resp);
-			delete resp;
-			return;
-		}
-
+		
+		assert(user);
+		user_config = user;
+		user_defined_ringtone = ringtone;
+		
 		call_info.from_uri = r->hdr_from.uri;
 		call_info.from_display = r->hdr_from.display;
+		call_info.from_display_override = r->hdr_from.display_override;
 		if (r->hdr_organization.is_populated()) {
 			call_info.from_organization = r->hdr_organization.name;
 		} else {
@@ -1058,6 +1257,9 @@ void t_line::recvd_invite(t_request *r, t_tid tid) {
 		ui->cb_line_state_changed();
 		start_timer(LTMR_NO_ANSWER);
 		cleanup();
+		
+		// Answer if auto answer mode is activated
+		if (auto_answer) answer();
 		break;
 	case LS_BUSY:
 		// Only re-INVITEs can be sent to a busy line
@@ -1290,8 +1492,15 @@ void t_line::timeout(t_line_timer timer, t_dialog_id did) {
 		// Reject call or redirect it if CF_NOANSWER is active.
 		// If there is no active dialog then ignore the timeout.
 		// The timer should have been stopped already.
+		log_file->write_report("No answer timeout",
+					"t_line::timeout");
+		
 		if (active_dialog) {
-			if (phone->service.get_cf_active(CF_NOANSWER, cf_dest)) {
+			assert(user_config);
+			t_service *srv = phone->ref_service(user_config);
+			if (srv->get_cf_active(CF_NOANSWER, cf_dest)) {
+				log_file->write_report("Call redirection no answer",
+					"t_line::timeout");
 				active_dialog->redirect(cf_dest,
 					R_302_MOVED_TEMPORARILY);
 			} else {
@@ -1331,6 +1540,13 @@ void t_line::timeout(t_line_timer timer, t_dialog_id did) {
 			dialog->timeout(timer);
 		}
 		break;
+	case LTMR_CANCEL_GUARD:
+		// If there is no dialog then ignore the timeout
+		if (dialog) {
+			dialog->id_cancel_guard = 0;
+			dialog->timeout(timer);
+		}
+		break;
 	default:
 		assert(false);
 	}
@@ -1418,16 +1634,19 @@ void t_line::process_invite_retrans(void) {
 }
 
 string t_line::create_user_contact(void) const {
-	return phone->create_user_contact();
+	assert(user_config);
+	return user_config->create_user_contact();
 }
 
 string t_line::create_user_uri(void) const {
-	return phone->create_user_uri();
+	assert(user_config);
+	return user_config->create_user_uri();
 }
 
 t_response *t_line::create_options_response(t_request *r, bool in_dialog) const
 {
-	return phone->create_options_response(r, in_dialog);
+	assert(user_config);
+	return phone->create_options_response(user_config, r, in_dialog);
 }
 
 void t_line::send_response(t_response *r, t_tuid tuid, t_tid tid) {
@@ -1435,7 +1654,8 @@ void t_line::send_response(t_response *r, t_tuid tuid, t_tid tid) {
 }
 
 void t_line::send_request(t_request *r, t_tuid tuid) {
-	phone->send_request(r, tuid);
+	assert(user_config);
+	phone->send_request(user_config, r, tuid);
 }
 
 t_phone *t_line::get_phone(void) const {
@@ -1454,6 +1674,14 @@ bool t_line::get_is_muted(void) const {
 	return is_muted;
 }
 
+bool t_line::get_auto_answer(void) const {
+	return auto_answer;
+}
+
+void t_line::set_auto_answer(bool enable) {
+	auto_answer = enable;
+}
+
 bool t_line::is_refer_succeeded(void) const {
 	if (active_dialog) return active_dialog->refer_succeeded;
 	return false;
@@ -1477,6 +1705,12 @@ void t_line::unseize(void) {
 	ui->cb_line_state_changed();
 }
 
+t_session *t_line::get_session(void) const {
+	if (!active_dialog) return NULL;
+
+	return active_dialog->get_session();
+}
+
 t_audio_session *t_line::get_audio_session(void) const {
 	if (!active_dialog) return NULL;
 
@@ -1509,8 +1743,9 @@ t_call_info t_line::get_call_info(void) const {
 	return call_info;
 }
 
-void t_line::ci_set_dtmf_supported(bool supported) {
+void t_line::ci_set_dtmf_supported(bool supported, bool inband) {
 	call_info.dtmf_supported = supported;
+	call_info.dtmf_inband = inband;
 }
 
 void t_line::ci_set_last_provisional_reason(const string &reason) {
@@ -1530,10 +1765,29 @@ void t_line::ci_set_refer_supported(bool supported) {
 }
 
 void t_line::init_rtp_port(void) {
-	rtp_port = user_config->rtp_port + line_number * 2;
+	rtp_port = sys_config->get_rtp_port() + line_number * 2;
 }
 
 unsigned short t_line::get_rtp_port(void) const {
 	return rtp_port;
 }
 
+t_user *t_line::get_user(void) const {
+	return user_config;
+}
+
+string t_line::get_ringtone(void) const {
+	if (!user_defined_ringtone.empty()) {
+		// Ring tone returned by incoming call script
+		return user_defined_ringtone;
+	} else if (!user_config->get_ringtone_file().empty()) {
+		// Ring tone from user profile
+		return user_config->get_ringtone_file();
+	} else if (!sys_config->get_ringtone_file().empty()) {
+		// Ring tone from system settings
+		return sys_config->get_ringtone_file();
+	} else {
+		// Twinkle default
+		return FILE_RINGTONE;
+	}	
+}
