@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,6 +38,17 @@ string t_hdr_content_encoding::encode(void) const {
 		s = "Content-Encoding: ";
 	}
 
+	s += encode_value();
+	s += CRLF;
+
+	return s;
+}
+
+string t_hdr_content_encoding::encode_value(void) const {
+	string s;
+
+	if (!populated) return s;
+
 	for (list<t_coding>::const_iterator i = coding_list.begin();
 	     i != coding_list.end(); i++)
 	{
@@ -45,7 +56,14 @@ string t_hdr_content_encoding::encode(void) const {
 		s += i->encode();
 	}
 
-	s += CRLF;
+	return s;
+}
 
+string t_hdr_content_encoding::encode_env(void) const {
+	string s;
+	
+	s = "SIP_CONTENT_ENCODING=";
+	s += encode_value();
+	
 	return s;
 }
