@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,6 +33,8 @@ public:
 	void set_date_gm(struct tm *tm); // set date, tm is GMT
 	void set_now(void); // Set date/time to current date/time
 	string encode(void) const;
+	string encode_value(void) const;
+	string encode_env(void) const;
 };
 
 using namespace std;
