@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 
 #include "definitions.h"
 #include "hdr_record_route.h"
+#include "parse_ctrl.h"
 
 void t_route::add_param(const t_parameter &p) {
 	params.push_back(p);
@@ -53,11 +54,41 @@ void t_hdr_record_route::add_route(const t_route &r) {
 }
 
 string t_hdr_record_route::encode(void) const {
+	return (t_parser::multi_values_as_list ? encode_list() : encode_multi_header());
+}
+
+string t_hdr_record_route::encode_list(void) const {
 	string s;
 
 	if (!populated) return s;
 
 	s = "Record-Route: ";
+	s += encode_value();
+	s += CRLF;
+	
+	return s;
+}
+
+string t_hdr_record_route::encode_multi_header(void) const {
+	string s;
+
+	if (!populated) return s;
+
+	for (list<t_route>::const_iterator i = route_list.begin();
+	     i != route_list.end(); i++)
+	{
+		s += "Record-Route: ";
+		s += i->encode();
+		s += CRLF;
+	}
+
+	return s;
+}
+
+string t_hdr_record_route::encode_value(void) const {
+	string s;
+
+	if (!populated) return s;
 
 	for (list<t_route>::const_iterator i = route_list.begin();
 	     i != route_list.end(); i++)
@@ -66,6 +97,15 @@ string t_hdr_record_route::encode(void) const {
 		s += i->encode();
 	}
 
-	s += CRLF;
 	return s;
 }
+
+string t_hdr_record_route::encode_env(void) const {
+	string s;
+
+	s = "SIP_RECORD_ROUTE=";
+	s += encode_value();
+	
+	return s;
+}
+	
\ No newline at end of file
