@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -46,7 +46,23 @@ string t_hdr_user_agent::encode(void) const {
 	if (!populated) return s;
 
 	s = "User-Agent: ";
-	s += get_ua_info();
+	s += encode_value();
 	s += CRLF;
+	
+	return s;
+}
+
+string t_hdr_user_agent::encode_value(void) const {
+	if (!populated) return "";
+
+	return get_ua_info();
+}
+
+string t_hdr_user_agent::encode_env(void) const {
+	string s;
+	
+	s = "SIP_USER_AGENT=";
+	s += encode_value();
+	
 	return s;
 }
