@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,10 +21,12 @@
 
 #include <list>
 #include <string>
+#include "events.h"
 #include "phone.h"
 #include "parser/request.h"
 #include "parser/response.h"
 #include "audio/tone_gen.h"
+#include "threads/thread.h"
 
 #include "twinkle_config.h"
 
@@ -53,11 +55,14 @@ struct t_command_arg {
         string  value;
 };
 
-class t_userintf {
+class t_userintf : public i_prohibit_thread {
 private:
         bool            end_interface; // indicates if interface loop should quit
         list<string>    all_commands;  // list of all commands
         t_tone_gen      *tone_gen;     // tone generator for ringing
+        
+        // The user for which out-of-dialog requests are executed.
+        t_user		*active_user;
 
         // The user can type a prefix of the command only. This method
         // completes a prefix to a full command.
@@ -72,29 +77,38 @@ private:
 
         // The command_list must contain the command itself as first
         // argument. Subsequent elements are the arguments.
-        bool exec_invite(const list<string> command_list);
+        bool exec_invite(const list<string> command_list, bool immediate = false);
+        bool exec_redial(const list<string> command_list);
         bool exec_answer(const list<string> command_list);
         bool exec_reject(const list<string> command_list);
-	bool exec_redirect(const list<string> command_list);
+	bool exec_redirect(const list<string> command_list, bool immediate = false);
 	bool exec_dnd(const list<string> command_list);
 	bool exec_auto_answer(const list<string> command_list);
         bool exec_bye(const list<string> command_list);
         bool exec_hold(const list<string> command_list);
         bool exec_retrieve(const list<string> command_list);
-	bool exec_refer(const list<string> command_list);
+	bool exec_refer(const list<string> command_list, bool immediate = false);
 	bool exec_conference(const list<string> command_list);
 	bool exec_mute(const list<string> command_list);
 	bool exec_dtmf(const list<string> command_list);
         bool exec_register(const list<string> command_list);
         bool exec_deregister(const list<string> command_list);
         bool exec_fetch_registrations(const list<string> command_list);
-        bool exec_options(const list<string> command_list);
+        bool exec_options(const list<string> command_list, bool immediate = false);
         bool exec_line(const list<string> command_list);
+        bool exec_user(const list<string> command_list);
         bool exec_quit(const list<string> command_list);
         bool exec_help(const list<string> command_list);
 
 protected:
         t_phone         *phone;
+        
+        // Asynchronous event queue
+        t_event_queue	evq_ui_events;
+        t_thread	*thr_process_events;
+        
+        // Indicates if commands should print output to stdout
+        bool		use_stdout;
 
 	// Throttle dtmtf not supported messages
 	bool		throttle_dtmf_not_supported;
@@ -103,6 +117,34 @@ protected:
 	t_url		last_called_url;
 	string		last_called_display;
 	string		last_called_subject;
+	string		last_called_profile; // profile used to make the call
+	
+	// The do_* methods perform the commands parsed by the exec_* methods.
+	virtual bool do_invite(const string &destination, const string &display, 
+			const string &subject, bool immediate);
+	virtual void do_redial(void);
+	virtual void do_answer(void);
+	virtual void do_reject(void);
+	virtual void do_redirect(bool show_status, bool type_present, t_cf_type cf_type, 
+		bool action_present, bool enable, int num_redirections,
+		const list<string> &dest_strlist, bool immediate);
+	virtual void do_dnd(bool show_status, bool toggle, bool enable);
+	virtual void do_auto_answer(bool show_status, bool toggle, bool enable);
+	virtual void do_bye(void);
+	virtual void do_hold(void);
+	virtual void do_retrieve(void);
+	virtual bool do_refer(const string &destination, bool immediate);
+	virtual void do_conference(void);
+	virtual void do_mute(bool show_status, bool toggle, bool enable);
+	virtual void do_dtmf(const string &digits);
+	virtual void do_register(bool reg_all_profiles);
+	virtual void do_deregister(bool dereg_all_profiles, bool dereg_all_devices);
+	virtual void do_fetch_registrations(void);
+	virtual bool do_options(bool dest_set, const string &destination, bool immediate);
+	virtual void do_line(int line);
+	virtual void do_user(const string &profile_name);
+	virtual void do_quit(void);
+	virtual void do_help(const list<t_command_arg> &al);
 
 public:
         t_userintf(t_phone *_phone);
@@ -110,14 +152,25 @@ public:
 
         // Expand a SIP destination to a full SIP uri, i.e. add sip: scheme
         // and domain if these are missing.
-        string expand_destination(const string &dst);
+        string expand_destination(t_user *user_config, const string &dst);
         
         // Expand a SIP destination into a display and a full SIP uri
-        void expand_destination(const string &dst, string &display, string &dst_url);
-        void expand_destination(const string &dst, t_display_url &display_url);
+        void expand_destination(t_user *user_config, 
+        	const string &dst, string &display, string &dst_url);
+        void expand_destination(t_user *user_config, 
+        	const string &dst, t_display_url &display_url);
+        	
+        // Expand a SIP destination as above, but split of any headers if any.
+        // If the subject header is present, then its value will be returned in
+        // subject.
+        // The dst_no_headers parameter will contain the dst string with the headers
+        // cut off.
+	void expand_destination(t_user *user_config,
+		const string &dst, t_display_url &display_url, string &subject,
+		string &dst_no_headers);
 
 	// Format a SIP address for user display
-	virtual string format_sip_address(const string &display,
+	virtual string format_sip_address(t_user *user_config, const string &display,
 	                                  const t_url &uri) const;
 
 	// Format a warning for user display
@@ -126,8 +179,20 @@ public:
 	// Format a codec for user display
 	virtual string format_codec(t_audio_codec codec) const;
 
-        bool exec_command(const string &command_line);
+	// The immediate flag is by the cmd_cli method (see below)
+        bool exec_command(const string &command_line, bool immediate = false);
+        
+        // Run the user interface
         virtual void run(void);
+        
+        // This method executes asynchronous uier interface events
+        virtual void process_events(void);
+        
+        // Save user interface state to system settings
+        virtual void save_state(void);
+        
+        // Restore user interface state from system settings
+        virtual void restore_state(void);
 
 	// Lock the user interface to synchornize output
 	virtual void lock(void);
@@ -137,10 +202,10 @@ public:
         virtual string select_network_intf(void);
 
 	// Select a user configuration file. Returns false if selection failed.
-	virtual bool select_user_config(string &config_file);
+	virtual bool select_user_config(list<string> &config_files);
 
         // Call back functions
-        virtual void cb_incoming_call(int line, const t_request *r);
+        virtual void cb_incoming_call(t_user *user_config, int line, const t_request *r);
         virtual void cb_call_cancelled(int line);
         virtual void cb_far_end_hung_up(int line);
         virtual void cb_answer_timeout(int line);
@@ -152,35 +217,46 @@ public:
 	virtual void cb_prack_failed(int line, const t_response *r);
         virtual void cb_provisional_resp_invite(int line, const t_response *r);
         virtual void cb_cancel_failed(int line, const t_response *r);
-        virtual void cb_call_answered(int line, const t_response *r);
-        virtual void cb_call_failed(int line, const t_response *r);
+        virtual void cb_call_answered(t_user *user_config, int line, const t_response *r);
+        virtual void cb_call_failed(t_user *user_config, int line, const t_response *r);
+        virtual void cb_stun_failed_call_ended(int line);
         virtual void cb_call_ended(int line, const t_response *r);
         virtual void cb_call_established(int line);
         virtual void cb_options_response(const t_response *r);
         virtual void cb_reinvite_success(int line, const t_response *r);
         virtual void cb_reinvite_failed(int line, const t_response *r);
 	virtual void cb_retrieve_failed(int line, const t_response *r);
-        virtual void cb_invalid_reg_resp(const t_response *r, const string &reason);
-        virtual void cb_register_success(const t_response *r, unsigned long expires,
-					 bool first_success);
-        virtual void cb_register_failed(const t_response *r, bool first_failure);
-        virtual void cb_register_stun_failed(bool first_failure);
-        virtual void cb_deregister_success(const t_response *r);
-        virtual void cb_deregister_failed(const t_response *r);
-        virtual void cb_fetch_reg_failed(const t_response *r);
-        virtual void cb_fetch_reg_result(const t_response *r);
-	virtual void cb_register_inprog(t_register_type register_type);
-	virtual void cb_redirecting_request(int line, const t_contact_param &contact);
-	virtual void cb_redirecting_request(const t_contact_param &contact);
-        virtual void cb_play_ringtone(void);
-	virtual void cb_play_ringback(void);
+        virtual void cb_invalid_reg_resp(t_user *user_config, 
+        		const t_response *r, const string &reason);
+        virtual void cb_register_success(t_user *user_config, 
+        		const t_response *r, unsigned long expires, bool first_success);
+        virtual void cb_register_failed(t_user *user_config, 
+        		const t_response *r, bool first_failure);
+        virtual void cb_register_stun_failed(t_user *user_config, bool first_failure);
+        virtual void cb_deregister_success(t_user *user_config, const t_response *r);
+        virtual void cb_deregister_failed(t_user *user_config, const t_response *r);
+        virtual void cb_fetch_reg_failed(t_user *user_config, const t_response *r);
+        virtual void cb_fetch_reg_result(t_user *user_config, const t_response *r);
+	virtual void cb_register_inprog(t_user *user_config, t_register_type register_type);
+	virtual void cb_redirecting_request(t_user *user_config, 
+			int line, const t_contact_param &contact);
+	virtual void cb_redirecting_request(t_user *user_config, 
+			const t_contact_param &contact);
+        virtual void cb_play_ringtone(int line);
+	virtual void cb_play_ringback(t_user *user_config);
         virtual void cb_stop_tone(int line);
+        virtual void cb_notify_call(int line, string from_party);
+        virtual void cb_stop_call_notification(int line);
 	virtual void cb_dtmf_detected(int line, char dtmf_event);
+	virtual void cb_async_dtmf_detected(int line, char dtmf_event);
+	virtual void cb_send_dtmf(int line, char dtmf_event);
+	virtual void cb_async_send_dtmf(int line, char dtmf_event);
 	virtual void cb_dtmf_not_supported(int line);
 	virtual void cb_dtmf_supported(int line);
 	virtual void cb_line_state_changed(void);
 	virtual void cb_send_codec_changed(int line, t_audio_codec codec);
 	virtual void cb_recv_codec_changed(int line, t_audio_codec codec);
+	virtual void cb_async_recv_codec_changed(int line, t_audio_codec codec);
 	virtual void cb_notify_recvd(int line, const t_request *r);
 	virtual void cb_refer_failed(int line, const t_response *r);
 	virtual void cb_refer_result_success(int line);
@@ -188,23 +264,24 @@ public:
 	virtual void cb_refer_result_inprog(int line);
 
 	// A call is being referred by the far end. r must be the REFER request.
-	virtual void cb_call_referred(int line, t_request *r);
+	virtual void cb_call_referred(t_user *user_config, int line, t_request *r);
 
 	// The reference failed. Call to referrer is retrieved.
-	virtual void cb_retrieve_referrer(int line);
+	virtual void cb_retrieve_referrer(t_user *user_config, int line);
 	
 	// STUN errors
 	virtual void cb_stun_failed(int err_code, const string &err_reason);
 	virtual void cb_stun_failed(void);
 
 	// Interactive call back functions
-	virtual bool cb_ask_user_to_redirect_invite(const t_url &destination,
-			const string &display);
-	virtual bool cb_ask_user_to_redirect_request(const t_url &destination,
-			const string &display, t_method method);
-	virtual bool cb_ask_credentials(const string &realm, string &username,
-			string &password);
-	virtual bool cb_ask_user_to_refer(const t_url &refer_to_uri,
+	virtual bool cb_ask_user_to_redirect_invite(t_user *user_config, 
+			const t_url &destination, const string &display);
+	virtual bool cb_ask_user_to_redirect_request(t_user *user_config, 
+			const t_url &destination, const string &display, t_method method);
+	virtual bool cb_ask_credentials(t_user *user_config, 
+			const string &realm, string &username, string &password);
+	virtual bool cb_ask_user_to_refer(t_user *user_config, 
+			const t_url &refer_to_uri,
 			const string &refer_to_display,
 			const t_url &referred_by_uri,
 			const string &referred_by_display);
@@ -222,15 +299,36 @@ public:
 	
 	// Call history has been updated
 	virtual void cb_call_history_updated(void);
+	virtual void cb_missed_call(int num_missed_calls);
+	
+	// Show firewall/NAT discovery progress
+	virtual void cb_nat_discovery_progress_start(int num_steps);
+	virtual void cb_nat_discovery_progress_step(int step);
+	virtual void cb_nat_discovery_finished(void);
+	virtual bool cb_nat_discovery_cancelled(void);
 
 	// Get last call information
 	// Returns true if last call information is valid
 	// Returns false is there is no valid last call information
 	virtual bool get_last_call_info(t_url &url, string &display,
-				string &subject) const;
+				string &subject, t_user **user_config) const;
 	virtual bool can_redial(void) const;
+	
+	// Execute external commands
+	// Some comments require confirmation from the user via the user
+	// interface, e.g. in GUI mode, a call dialog may popup for cmd_call.
+	// The 'immediate' flag indicates that no user confirmation is required.
+	// The command should be executed immediately.
+	virtual void cmd_call(const string &destination, bool immediate);
+	virtual void cmd_quit(void);
+	virtual void cmd_cli(const string &command, bool immeidate);
+	
+	// Lookup a URL in the address book
+	virtual string get_name_from_abook(t_user *user_config, const t_url &u);
 };
 
+void *process_events_main(void *arg);
+
 extern t_userintf *ui;
 
 #endif
